from django.apps import apps as django_apps
from django.test import TestCase
from edc_lab.models import Aliquot, Box, BoxType
from edc_lab.models.box_item import BoxItem

from edc_lab_dashboard.model_wrappers import BoxModelWrapper

app_config = django_apps.get_app_config("edc_lab_dashboard")


class TestModelWrapper(TestCase):
    def setUp(self):
        self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
        self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
        self.box_item = BoxItem.objects.create(box=self.box, position=0)
        self.aliquot = Aliquot.objects.create(
            subject_identifier="ABCDEFG",
            count=1,
            is_primary=True,
            aliquot_type="Whole Blood",
            numeric_code="02",
            alpha_code="WB",
        )

        self.wrapper_cls = BoxModelWrapper
        box_type = BoxType.objects.create(across=9, down=9, total=81)
        self.box = Box.objects.create(box_identifier="1234", box_type=box_type)

    def test_box_model_wrapper_href(self):
        wrapper = self.wrapper_cls(self.box)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/box/{self.box.id}/change/?"
            f"next=edc_lab_dashboard:pack_listboard_url&",
        )

    def test_box_model_wrapper_template_attrs(self):
        wrapper = self.wrapper_cls(self.box)
        attrs = ["human_readable_identifier", "comment", "created", "user_created"]
        for attr in attrs:
            with self.subTest(attr=attr):
                self.assertTrue(hasattr(wrapper, attr))



from copy import copy

from django.apps import apps as django_apps
from django.test import TestCase
from edc_lab.models import Aliquot, Box, BoxItem, BoxType

from edc_lab_dashboard.model_wrappers import AliquotModelWrapper

app_config = django_apps.get_app_config("edc_lab_dashboard")


class TestModelWrapper(TestCase):
    def setUp(self):
        self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
        self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
        self.box_item = BoxItem.objects.create(box=self.box, position=0)
        self.aliquot = Aliquot.objects.create(
            subject_identifier="ABCDEFG",
            count=1,
            is_primary=True,
            aliquot_type="Whole Blood",
            numeric_code="02",
            alpha_code="WB",
        )
        self.wrapper_cls = AliquotModelWrapper

    def test_aliquot_model_wrapper(self):
        wrapper = self.wrapper_cls(self.aliquot)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/aliquot/{self.aliquot.id}/change/?"
            f"next=edc_lab_dashboard:aliquot_listboard_url&",
        )
        self.assertIn("edc_lab_dashboard:aliquot_listboard_url", wrapper.href)

    def test_aliquot_wrapper_attrs(self):
        """Asserts attrs used in template exist."""
        wrapper = self.wrapper_cls(self.aliquot)
        attrs = [
            "is_primary",
            "box_item",
            "manifest_item",
            "subject_identifier",
            "aliquot_datetime",
            "aliquot_type",
            "numeric_code",
            "alpha_code",
            "box_item",
            "shipped",
            "count",
        ]
        for attr in attrs:
            with self.subTest(attr=attr):
                self.assertTrue(hasattr(wrapper, attr))

    def test_aliquot_wrapper_attrs2(self):
        wrapper = self.wrapper_cls(self.aliquot)
        self.assertTrue(wrapper.human_readable_identifier)
        self.assertTrue(wrapper.is_primary)
        self.assertTrue(wrapper.box_item)
        self.assertTrue(wrapper.box_item.box.box_identifier)
        self.assertTrue(wrapper.box_item.box.get_category_display)
        self.assertTrue(wrapper.box_item.box.box_identifier)
        self.assertTrue(wrapper.subject_identifier)
        self.assertTrue(wrapper.aliquot_datetime)
        self.assertTrue(wrapper.aliquot_type)
        self.assertTrue(wrapper.numeric_code)
        self.assertTrue(wrapper.alpha_code)
        self.assertTrue(wrapper.box_item)
        self.assertTrue(wrapper.count)
        self.assertIsNone(wrapper.manifest_item)
        self.assertFalse(wrapper.shipped)

    def test_aliquot_wrapper_attrs3(self):
        aliquot = copy(self.aliquot)
        wrapper = self.wrapper_cls(aliquot)
        self.box_item.delete()
        self.assertIsNone(wrapper.box_item)

        aliquot = copy(self.aliquot)
        wrapper = self.wrapper_cls(aliquot)
        self.assertIsNone(wrapper.box_item)



from django.test import TestCase
from django.urls import reverse


class TestUrls(TestCase):
    def test_requisition_listboard_url(self):
        url = reverse("edc_lab_dashboard:requisition_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/requisition/", url)

        url = reverse("edc_lab_dashboard:requisition_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/requisition/1/", url)

    def test_receive_listboard_url(self):
        url = reverse("edc_lab_dashboard:receive_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/receive/", url)

        url = reverse("edc_lab_dashboard:receive_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/receive/1/", url)

    def test_process_listboard_url(self):
        url = reverse("edc_lab_dashboard:process_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/process/", url)

        url = reverse("edc_lab_dashboard:process_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/process/1/", url)

    def test_pack_listboard_url(self):
        url = reverse("edc_lab_dashboard:pack_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/pack/", url)

        url = reverse("edc_lab_dashboard:pack_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/pack/1/", url)

    def test_aliquot_listboard_url(self):
        url = reverse("edc_lab_dashboard:aliquot_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/aliquot/", url)

        url = reverse("edc_lab_dashboard:aliquot_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/aliquot/1/", url)

    def test_manifest_listboard_url(self):
        url = reverse("edc_lab_dashboard:manifest_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/manifest/", url)

        url = reverse("edc_lab_dashboard:manifest_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/manifest/1/", url)

    def test_result_listboard_url(self):
        url = reverse("edc_lab_dashboard:result_listboard_url")
        self.assertEqual("/edc_lab_dashboard/listboard/result/", url)

        url = reverse("edc_lab_dashboard:result_listboard_url", kwargs=dict(page=1))
        self.assertEqual("/edc_lab_dashboard/listboard/result/1/", url)

    def test_manage_box_listboard_url(self):
        url = reverse(
            "edc_lab_dashboard:manage_box_listboard_url",
            kwargs=dict(action_name="manage"),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/manage/", url)

        url = reverse(
            "edc_lab_dashboard:manage_box_listboard_url",
            kwargs=dict(action_name="manage", box_identifier="ABC123"),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/manage/ABC123/", url)

        url = reverse(
            "edc_lab_dashboard:manage_box_listboard_url",
            kwargs=dict(action_name="manage", box_identifier="ABC123", page=1),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/manage/ABC123/1/", url)

    def test_verify_box_listboard_url(self):
        url = reverse(
            "edc_lab_dashboard:verify_box_listboard_url",
            kwargs=dict(action_name="verify", position=22),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/verify/22/", url)

        url = reverse(
            "edc_lab_dashboard:verify_box_listboard_url",
            kwargs=dict(action_name="verify", box_identifier="ABC123", position=22),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/verify/ABC123/22/", url)

        url = reverse(
            "edc_lab_dashboard:verify_box_listboard_url",
            kwargs=dict(action_name="verify", box_identifier="ABC123", position=22, page=1),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/box/verify/ABC123/22/1/", url)

    def test_manage_manifest_listboard_url(self):
        url = reverse(
            "edc_lab_dashboard:manage_manifest_listboard_url",
            kwargs=dict(action_name="manage", manifest_identifier="ABC123", page=1),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/manifest/manage/ABC123/1/", url)

        url = reverse(
            "edc_lab_dashboard:manage_manifest_listboard_url",
            kwargs=dict(action_name="manage", manifest_identifier="ABC123"),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/manifest/manage/ABC123/", url)

        url = reverse(
            "edc_lab_dashboard:manage_manifest_listboard_url",
            kwargs=dict(action_name="manage"),
        )
        self.assertEqual("/edc_lab_dashboard/listboard/manifest/manage/", url)

    def test_manage_box_item_form_action_url(self):
        url = reverse(
            "edc_lab_dashboard:manage_box_item_form_action_url",
            kwargs=dict(action_name="manage", box_identifier="ABC123"),
        )
        self.assertEqual("/edc_lab_dashboard/box/ABC123/manage/", url)

    def test_verify_box_item_form_action_url(self):
        url = reverse(
            "edc_lab_dashboard:verify_box_item_form_action_url",
            kwargs=dict(action_name="verify", box_identifier="ABC123", position=22),
        )
        self.assertEqual("/edc_lab_dashboard/box/ABC123/verify/22/", url)

    def test_manage_manifest_item_form_action_url(self):
        url = reverse(
            "edc_lab_dashboard:manage_manifest_item_form_action_url",
            kwargs=dict(action_name="manage", manifest_identifier="ABC123"),
        )
        self.assertEqual("/edc_lab_dashboard/manifest/ABC123/manage/", url)

    def test_requisition_form_action_url(self):
        url = reverse("edc_lab_dashboard:requisition_form_action_url")
        self.assertEqual("/edc_lab_dashboard/requisition/", url)

    def test_receive_form_action_url(self):
        url = reverse("edc_lab_dashboard:receive_form_action_url")
        self.assertEqual("/edc_lab_dashboard/requisition/receive/", url)

    def test_process_url(self):
        url = reverse("edc_lab_dashboard:process_form_action_url")
        self.assertEqual("/edc_lab_dashboard/requisition/process/", url)

    def test_pack_form_action_url(self):
        url = reverse("edc_lab_dashboard:pack_form_action_url")
        self.assertEqual("/edc_lab_dashboard/requisition/pack/", url)

    def test_manifest_url(self):
        url = reverse("edc_lab_dashboard:manifest_form_action_url")
        self.assertEqual("/edc_lab_dashboard/manifest/", url)

    def test_aliquot_form_action_url(self):
        url = reverse("edc_lab_dashboard:aliquot_form_action_url")
        self.assertEqual("/edc_lab_dashboard/aliquot/", url)

    def test_home_url(self):
        url = reverse("edc_lab_dashboard:home_url")
        self.assertEqual("/edc_lab_dashboard/", url)
